@@ -2,6 +2,7 @@ from __future__ import absolute_import
 from __future__ import unicode_literals
 
 import functools
+import io
 import logging
 import os
 import string
@@ -67,7 +68,10 @@ DOCKER_CONFIG_KEYS = [
     'command',
     'cpu_count',
     'cpu_percent',
+    'cpu_period',
     'cpu_quota',
+    'cpu_rt_period',
+    'cpu_rt_runtime',
     'cpu_shares',
     'cpus',
     'cpuset',
@@ -125,11 +129,12 @@ ALLOWED_KEYS = DOCKER_CONFIG_KEYS + [
     'container_name',
     'credential_spec',
     'dockerfile',
+    'init',
     'log_driver',
     'log_opt',
     'logging',
     'network_mode',
-    'init',
+    'platform',
     'scale',
     'stop_grace_period',
 ]
@@ -1115,6 +1120,7 @@ def merge_build(output, base, override):
     md.merge_scalar('network')
     md.merge_scalar('target')
     md.merge_scalar('shm_size')
+    md.merge_scalar('isolation')
     md.merge_mapping('args', parse_build_arguments)
     md.merge_field('cache_from', merge_unique_items_lists, default=[])
     md.merge_mapping('labels', parse_labels)
@@ -1428,10 +1434,15 @@ def has_uppercase(name):
     return any(char in string.ascii_uppercase for char in name)
 
 
-def load_yaml(filename):
+def load_yaml(filename, encoding=None):
     try:
-        with open(filename, 'r') as fh:
+        with io.open(filename, 'r', encoding=encoding) as fh:
             return yaml.safe_load(fh)
-    except (IOError, yaml.YAMLError) as e:
+    except (IOError, yaml.YAMLError, UnicodeDecodeError) as e:
+        if encoding is None:
+            # Sometimes the user's locale sets an encoding that doesn't match
+            # the YAML files. Im such cases, retry once with the "default"
+            # UTF-8 encoding
+            return load_yaml(filename, encoding='utf-8')
         error_name = getattr(e, '__module__', '') + '.' + e.__class__.__name__
         raise ConfigurationError(u"{}: {}".format(error_name, e))
